@@ -1,8 +1,7 @@
 package util
 
 import (
-	"io/ioutil"
-
+	"os"
 	"gopkg.in/yaml.v2"
 )
 
@@ -34,6 +33,7 @@ type ClusterOpts struct {
 	ValuesFilePath       string `yaml:"valuesFilePath"`
 	DestinationNamespace string `yaml:"destinationNamespace"`
 	ClusterNamePrefix    string `yaml:"clusterNamePrefix"`
+	Concurrency 		 int 	`yaml:"parallel"`
 }
 
 type GenerateOpts struct {
@@ -45,8 +45,14 @@ type GenerateOpts struct {
 	Namespace       string `yaml:"namespace"`
 }
 
+func setDefaults(opts *GenerateOpts) {
+	if opts.ClusterOpts.Concurrency == 0 {
+		opts.ClusterOpts.Concurrency = 2
+	}
+}
+
 func Parse(opts *GenerateOpts, file string) error {
-	fp, err := ioutil.ReadFile(file)
+	fp, err := os.ReadFile(file)
 	if err != nil {
 		return err
 	}
@@ -55,5 +61,7 @@ func Parse(opts *GenerateOpts, file string) error {
 		return e
 	}
 
+	setDefaults(opts)
+
 	return nil
 }
